use std::sync::{
    mpsc,
    Mutex,
    Arc,

    PoisonError,
};
use std::fmt;
use std::error;

/// A multi-consumer message receiver
#[derive(Debug)]
pub struct PendingReceiver<T>
{
    recv: Arc<Mutex<mpsc::Receiver<T>>>,
}

impl<T> Clone for PendingReceiver<T>
{
    fn clone(&self) -> Self
    {
	Self{
	    recv: Arc::clone(&self.recv),
	}
    }
}

impl<T> PendingReceiver<T>
{
    /// Try to receive a message.
    pub fn recv(&self) -> Result<Option<T>, PendingReceiverError>
    {
	Ok(self.recv.lock()?.recv().ok())
    }
}

/// Create an `mpmc` channel.
pub fn channel<T>(cap: usize) -> (mpsc::SyncSender<T>, PendingReceiver<T>)
{
    let (tx, rx) = mpsc::sync_channel(cap);

    (tx, PendingReceiver{
	recv: Arc::new(Mutex::new(rx))
    })
}

#[derive(Debug)]
pub enum PendingReceiverError
{
    Poisoned,
    Closed,
}

impl error::Error for PendingReceiverError{}
impl fmt::Display for PendingReceiverError
{
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result
    {
	match self {
	    Self::Poisoned => write!(f, "poisoned"),
	    Self::Closed => write!(f, "receiver closed"),
	}
    }
}

impl<T> From<PoisonError<T>> for PendingReceiverError
{
    #[inline] fn from(_from: PoisonError<T>) -> Self
    {
	Self::Poisoned
    }
}

impl From<mpsc::RecvError> for PendingReceiverError
{
    fn from(_from: mpsc::RecvError) -> Self
    {
	Self::Closed
    }
}
